<?php

require_once 'Horde/Share.php';

/**
 * $Horde: trean/lib/Bookmarks.php,v 1.66 2006/01/01 21:12:01 jan Exp $
 *
 * Copyright 2004-2006 Ben Chavet <ben@horde.org>
 *
 * See the enclosed file LICENSE for license information (BSD). If you did not
 * receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * @author  Ben Chavet <ben@horde.org>
 * @package Trean
 */
class Trean_Bookmarks extends Horde_Share {

    var $_shareObject = 'DataTreeObject_BookmarkCategory';

    /**
     * Returns a new category object.
     *
     * @param string $name       The category's datatree name
     * @param array $properties  The category's initial properties. If set, a
     *                           'name' value is expected.
     *
     * @return DataTreeObject_BookmarkCategory  A new category object.
     */
    function &newCategory($name, $properties = null)
    {
        if (empty($name)) {
            $error = PEAR::raiseError(_("Category names must be non-empty"));
            return $error;
        }
        $category = &$this->newShare($name);
        $category->set('name', isset($properties['name']) ? $properties['name'] : '');
        return $category;
    }

    /**
     * Returns a new bookmark object.
     *
     * @param string $name       The bookmark's datatree name.
     * @param array $properties  The bookmark's initial properties. If set,
     *                           'url', 'title', and 'description' values are
     *                           expected.
     *
     * @return DataTreeObject_Bookmark  A new bookmark object.
     */
    function &newBookmark($name, $properties = null)
    {
        if (empty($name)) {
            $error = PEAR::raiseError(_("Bookmark names must be non-empty"));
            return $error;
        }
        $bookmark = &new DataTreeObject_Bookmark($name, $properties);
        $bookmark->setDataTree($this->_datatree);
        $bookmark->set('url', isset($properties['url']) ? $properties['url'] : '');
        $bookmark->set('title', isset($properties['title']) ? $properties['title'] : '');
        $bookmark->set('description', isset($properties['description']) ? $properties['description'] : '');
        return $bookmark;
    }

    /**
     * Returns a DataTreeObject_BookmarkCategory object corresponding to the
     * given ID.
     *
     * @param integer $id  The ID of the category to retrieve.
     *
     * @return DataTreeObject_BookmarkCategory  The category object
     *                                           corresponding to the given ID.
     */
    function &getCategory($id)
    {
        static $categoryCache = array();

        if (!isset($categoryCache[$id])) {
            $categoryCache[$id] = $this->_datatree->getObjectById($id, 'DataTreeObject_BookmarkCategory');
            if (!is_a($categoryCache[$id], 'PEAR_Error')) {
                $categoryCache[$id]->setShareOb($this);
            }
        }

        return $categoryCache[$id];
    }

    /**
     * Returns a DataTreeObject_Bookmark object corresponding to the given ID.
     *
     * @param integer $id  The ID of the bookmark to retrieve.
     *
     * @return DataTreeObject_Bookmark  The bookmark object corresponding to
     *                                  the given ID.
     */
    function &getBookmark($id)
    {
        static $bookmarkCache = array();

        if (!isset($bookmarkCache[$id])) {
            $bookmarkCache[$id] = $this->_datatree->getObjectById($id, 'DataTreeObject_Bookmark');
        }

        return $bookmarkCache[$id];
    }

    /**
     * Lists the bookmarks in this category.
     *
     * @param integer $from   The bookmark to start fetching.
     * @param integer $count  The numer of bookmarks to return.
     */
    function &listBookmarks($parent = DATATREE_ROOT, $allLevels = false,
                            $from = 0, $count = 0, $sortby = 'title',
                            $direction = 0, $extra_criteria = null)
    {
        $criteria = array('AND' => array(
                              array('field' => 'name',
                                    'op' => '=',
                                    'test' => 'type'),
                              array('field' => 'value',
                                    'op' => '=',
                                    'test' => 'bookmark')));

        if ($extra_criteria) {
            $criteria['AND']['JOIN'] = $extra_criteria;
            $criteria = array('AND' => $criteria);
        }

        $ids = $this->_datatree->getByAttributes($criteria, $parent, $allLevels, 'name', $from, $count, $sortby, null, $direction);
        if (is_a($ids, 'PEAR_Error') || !count($ids)) {
            return $ids;
        }

        $result = $this->_datatree->getObjects(array_keys($ids), 'DataTreeObject_Bookmark');
        return $result;
    }

    /**
     * Returns the number of bookmarks in all categories.
     *
     * @return integer  The number of all bookmarks.
     */
    function countBookmarks()
    {
        $criteria = array('AND' => array(
                              array('field' => 'name',
                                    'op' => '=',
                                    'test' => 'type'),
                              array('field' => 'value',
                                    'op' => '=',
                                    'test' => 'bookmark')));

        return $this->_datatree->countByAttributes($criteria);
    }

    /**
     * Adds a DataTreeObject_BookmarkCategory to the datatree backend.
     *
     * @param DataTreeObject_BookmarkCategory $category  The category to add.
     */
    function addCategory($category)
    {
        if (!is_a($category, 'DataTreeObject_BookmarkCategory')) {
            return PEAR::raiseError('Categories must be DataTreeObject_BookmarkCategory objects or extend that class.');
        }
        return $this->addShare($category);
    }

    /**
     * Adds a DataTreeObject_Bookmark to the datatree backend.
     *
     * @param DataTreeObject_Bookmark $bookmark  The bookmark to add.
     */
    function addBookmark($bookmark)
    {
        if (!is_a($bookmark, 'DataTreeObject_Bookmark')) {
            return PEAR::raiseError('Bookmarks must be DataTreeObject_Bookmark objects or extend that class.');
        }
        return $this->_datatree->add($bookmark);
    }

    /**
     * Removes a DataTreeObject_BookmarkCategory from the datatree backend.
     *
     * @param DataTreeObject_BookmarkCategory $category  The category to
     *                                                   remove.
     * @param boolean $force                             Force the removal of
     *                                                   every child?
     */
    function removeCategory($category, $force = false)
    {
        if (!is_a($category, 'DataTreeObject_BookmarkCategory')) {
            return PEAR::raiseError('Categories must be DataTreeObject_BookmarkCategory objects or extend that class.');
        }

        $result = Horde::callHook('_horde_hook_share_remove', array($category),
                                  'horde', false);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        return $this->_datatree->remove($category, $force);
    }

    /**
     * Removes a DataTreeObject_Bookmark from the datatree backend and
     * the favicon from VFS.
     *
     * @param DataTreeObject_Bookmark $bookmark  The bookmark to remove.
     * @param boolean $force                     Force to remove every child?
     */
    function removeBookmark($bookmark, $force = false)
    {
        if (!is_a($bookmark, 'DataTreeObject_Bookmark')) {
            return PEAR::raiseError('Bookmarks must be DataTreeObject_Bookmark objects or extend that class.');
        }
        
        $result = $this->_datatree->remove($bookmark, $force);

        if (!is_a($result, 'PEAR_Error')) {

            // Initialize VFS.
            require_once 'VFS.php';
            $vfs_params = Horde::getVFSConfig('favicons');
            if (!is_a($vfs_params, 'PEAR_Error')) {
                $vfs = &VFS::singleton($vfs_params['type'], $vfs_params['params']);
            }

            // Delete favicon from VFS
            if ($vfs->exists('.horde/trean/favicons/', $bookmark->get('favicon'))) {
                $del_f = $vfs->deleteFile('.horde/trean/favicons/', $bookmark->get('favicon'));
                if (is_a($del_f,"PEAR_Error")) {
                    $entry = "Could not delete favicon file .horde/trean/favicons/" . $bookmark->get('favicon');
                    Horde::logMessage($entry, __FILE__, __LINE__, PEAR_LOG_Error);
                }
            }

        }

        return $result;
    }

    /**
     * Returns a list of bookmarks matching the specified criteria.
     *
     * @param array $criteria  Criteria to search with.
     */
    function searchAllBookmarks($criteria)
    {
        return $this->_datatree->getByAttributes($criteria, Auth::getAuth(), true, false);
    }

    /**
     * Returns the datatree name for the given ID.
     *
     * @param integer $id  A DataTree ID.
     *
     * @return string  A DataTree name.
     */
    function getName($id)
    {
        return $this->_datatree->getName($id);
    }

    /**
     * Returns the datatree ID for the given nam.
     *
     * @param string $name  A DataTree name.
     *
     * @return integer  A DataTree ID.
     */
    function getId($name)
    {
        return $this->_datatree->getId($name);
    }

    /**
     * Returns the datatree parent of the object with the given ID.
     */
    function getParent($id)
    {
        return $this->_datatree->getParent($id);
    }

    function getParents($id, $getids = false)
    {
        return $this->_datatree->getParents($id, $getids);
    }

    function extractOneLevel($parent)
    {
        return $this->_datatree->_extractOneLevel($parent);
    }

    function move($object, $category = null)
    {
        if (!is_a($category, 'DataTreeObject_BookmarkCategory')) {
            return PEAR::raiseError('Categories must be DataTreeObject_BookmarkCategory objects or extend that class.');
        }
        return $this->_datatree->move($object, $category);
    }

    function copy($bookmark, $category)
    {
        if (!is_a($category, 'DataTreeObject_BookmarkCategory')) {
            return PEAR::raiseError('Categories must be DataTreeObject_BookmarkCategory objects or extend that class.');
        }

        if (!is_a($bookmark, 'DataTreeObject_Bookmark')) {
            return PEAR::raiseError('Bookmarks must be DataTreeObject_Bookmark objects or extend that class.');
        }

        $properties = array('url' => $bookmark->get('url'),
                            'title' => $bookmark->get('title'),
                            'description' => $bookmark->get('description'));

        return $category->addBookmark($properties);
    }
}

/**
 * Extension of the DataTreeObject class for storing Bookmark Categories in
 * the DataTree structure.
 *
 * @author  Ben Chavet <ben@horde.org>
 * @package Trean
 */
class DataTreeObject_BookmarkCategory extends DataTreeObject_Share {

    function DataTreeObject_BookmarkCategory($name)
    {
        parent::DataTreeObject_Share($name);
        $this->data['type'] = 'category';
    }

    /**
     * Sets one of the attributes of the object.
     *
     * @param string   The attribute to set.
     * @param mixed    The value for $attribute.
     * @param boolean  Determines whether the backend should be updated or not.
     */
    function set($attribute, $value, $update = false)
    {
        parent::set($attribute, $value);
        if ($update) {
            $this->update();
        }
    }

    /**
     * Adds a bookmark to this category.
     *
     * @param array $properties  The initial properties for the new bookmark.
     *                           Expected values are 'url', 'title', and
     *                           'description'.
     *
     * @return  The datatree ID for the new bookmark.
     */
    function addBookmark($properties)
    {
        if ($this->getId() == DATATREE_ROOT) {
            $bookmark = &$this->_shareOb->newBookmark(md5(uniqid(mt_rand())), $properties);
        } else {
            $bookmark = &$this->_shareOb->newBookmark($this->getName() . ':' . md5(uniqid(mt_rand())), $properties);
        }
        $this->_shareOb->addBookmark($bookmark);
        return $this->datatree->getId($bookmark);
    }

    /**
     * Adds a child category to this category.
     *
     * @param array $properties  The initial properties for the new category.
     *                           Expected value is 'name'.
     *
     * @return  The datatree ID for the new category.
     */
    function addCategory($properties)
    {
        if ($this->getId() == DATATREE_ROOT) {
            $category = &$this->_shareOb->newCategory(md5(uniqid(mt_rand())), $properties);
        } else {
            $category = &$this->_shareOb->newCategory($this->getName() . ':' . md5(uniqid(mt_rand())), $properties);
        }
        $this->_shareOb->addCategory($category);
        return $this->datatree->getId($category);
    }

    /**
     * Returns the datatree ID for this object.
     */
    function getId()
    {
        return $this->datatree->getId($this);
    }

    /**
     * Returns the datatree parent of this object.
     */
    function getParent()
    {
        return $this->datatree->getParent($this->getName());
    }

    /**
     * Updates the datatree backend with the current values of this category.
     */
    function update()
    {
        return $this->datatree->updateData($this);
    }

    /**
     * Deletes this category from the datatree backend.
     */
    function delete()
    {
        $this->_shareOb->removeCategory($this, true);
    }

    function &listBookmarks($allLevels = false, $from = 0, $count = 0)
    {
        $list = $this->_shareOb->listBookmarks($this->getName(), $allLevels, $from, $count);
        return $list;
    }

    /**
     * Map this object's attributes from the data array into a format that we
     * can store in the attributes storage backend.
     *
     * @return array  The attributes array.
     */
    function _toAttributes()
    {
        $attributes = parent::_toAttributes(true);

        foreach ($this->data as $key => $value) {
            if ($key == 'perm') {
                continue;
            } else {
                $attributes[] = array('name' => $key,
                                      'key' => '',
                                      'value' => $value);
            }
        }

        return $attributes;
    }

    /**
     * Takes in a list of attributes from the backend and maps it to our
     * internal data array.
     *
     * @param array $attributes  The list of attributes from the backend
     *                           (attribute name, key, and value).
     */
    function _fromAttributes($attributes)
    {
        parent::_fromAttributes($attributes, true);

        foreach ($attributes as $attr) {
            if (substr($attr['name'], 0, 4) == 'perm') {
                continue;
            } else {
                $this->data[$attr['name']] = $attr['value'];
            }
        }
    }

}

/**
 * @author  Ben Chavet <ben@horde.org>
 * @package Trean
 */
class DataTreeObject_Bookmark extends DataTreeObject {

    function DataTreeObject_Bookmark($name, $properties = null)
    {
        parent::DataTreeObject($name);
        $this->data['type'] = 'bookmark';
        $this->data['clicks'] = 0;
        $this->data['rating'] = 0;
    }

    /**
     * Sets one of the attributes of the object.
     *
     * @param string   The attribute to set.
     * @param mixed    The value for $attribute.
     * @param boolean  Determines whether the backend should be updated or not.
     */
    function set($attribute, $value, $update = false)
    {
        parent::set($attribute, $value);
        if ($update) {
            $this->update();
        }
    }

    /**
     * Returns the datatree ID for this object.
     */
    function getId()
    {
        return $this->datatree->getId($this);
    }

    /**
     * Returns the datatree parent of this object.
     */
    function getParent()
    {
        return $this->datatree->getParent($this->getName());
    }

    /**
     * Updates the datatree backend with the current values of this bookmark.
     */
    function update()
    {
        return $this->datatree->updateData($this);
    }

    /**
     * Deletes this bookmark from the datatree backend.
     */
    function delete()
    {
        return $GLOBALS['trean_shares']->removeBookmark($this);
    }

    /**
     * Moves this bookmark to a new category.
     *
     * @param DataTreeObject_BookmarkCategory $newParent  The new parent of
     *                                                    this bookmark.
     */
    function move($newParent)
    {
        return $GLOBALS['trean_shares']->_datatree->move($this, $newParent);
    }

    /**
     * Increments this bookmarks click count.
     */
    function addClick($update = true)
    {
        $this->data['clicks']++;

        if ($update) {
            $this->update();
        }
    }

    /**
     * Maps this object's attributes from the data array into a format that we
     * can store in the attributes storage backend.
     *
     * @return array  The attributes array.
     */
    function _toAttributes()
    {
        foreach ($this->data as $key => $value) {
            $attributes[] = array('name' => $key,
                                  'key' => '',
                                  'value' => $value);
        }
        return $attributes;
    }

    /**
     * Takes in a list of attributes from the backend and maps it to our
     * internal data array.
     *
     * @param array $attributes  The list of attributes from the backend
     *                           (attribute name, key, and value).
     */
    function _fromAttributes($attributes)
    {
        foreach ($attributes as $attribute) {
            $this->data[$attribute['name']] = $attribute['value'];
        }
    }

}
