<?php
/**
 * Trean Base Class.
 *
 * $Horde: trean/lib/Trean.php,v 1.65 2006/02/01 06:18:31 ben Exp $
 *
 * Copyright 2002-2006 Mike Cochrane <mike@graftonhall.co.nz>
 *
 * See the enclosed file LICENSE for license information (BSD). If you did not
 * receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @package Trean
 */
class Trean {

    /**
     * List categories.
     *
     * @return array  A list of categories.
     */
    function &listCategories($perm = PERMS_SHOW, $attributes = null,
                             $parent = DATATREE_ROOT, $allLevels = true,
                             $from = 0, $count = 0)
    {
        global $trean_shares, $notification;
        $categories = $trean_shares->listShares(Auth::getAuth(), $perm,
                                                $attributes, $parent,
                                                $allLevels, $from, $count,
                                                'name');
        if (is_a($categories, 'PEAR_Error')) {
            $notification->push(sprintf(_("An error occured listing categories: %s"), $categories->getMessage()), 'horde.error');
            return array();
        }
        return $categories;
    }

    /**
     * Counts the number of current user's categories list from storage.
     *
     * @param integer $perm       The level of permissions to require for a
     *                            category to return it.
     * @param mixed $attributes   Restrict the categories counted to those
     *                            matching $attributes. An array of
     *                            attribute/values pairs or a category owner
     *                            username.
     * @param string  $parent     The parent share to start searching at.
     * @param boolean $allLevels  Return all levels, or just the direct
     *                            children of $parent? Defaults to all levels.
     *
     * @return integer  The number of matching categories.
     */
    function countCategories($perm = PERMS_SHOW, $attributes = null,
                             $parent = DATATREE_ROOT, $allLevels = true)
    {
        global $trean_shares, $notification;
        $categories = $trean_shares->countShares(Auth::getAuth(), $perm,
                                                 $attributes, $parent,
                                                 $allLevels);
        if (is_a($categories, 'PEAR_Error')) {
            $notification->push(sprintf(_("An error occurred counting categories: %s"), $categories->getMessage()), 'horde.error');
            return 0;
        }
        return $categories;
    }

    /**
     * Lists all bookmarks in $parent or any of its children..
     *
     * @return array  A list of categories.
     */
    function listAllBookmarks($parent = DATATREE_ROOT, $depth = -1)
    {
        static $lists;
        if (is_null($lists)) {
            $lists = array();
        }

        $key = serialize(array($parent, $depth));
        if (!isset($lists[$key])) {
            $pcat = $GLOBALS['trean_shares']->getName($parent);
            if (is_a($pcat, 'PEAR_Error')) {
                $GLOBALS['notification']->push(_("Category does not exist."), 'horde.error');
                return array();
            }

            $result = $GLOBALS['trean_shares']->_datatree->get(DATATREE_FORMAT_FLAT, $pcat, true, _("My Bookmarks"), $depth);
            if (is_a($result, 'PEAR_Error')) {
                $GLOBALS['notification']->push($result, 'horde.error');
                return array();
            }

            $lists[$key] = array();
            foreach ($result as $id => $name) {
                if ($id == $parent) {
                    continue;
                }

                $object = &$GLOBALS['trean_shares']->getBookmark($id);
                if ($object->get('type') == 'category') {
                    continue;
                }

                $lists[$key][$id] = $name;
            }
        }

        return $lists[$key];
    }

    function listCategoryLevel($parent = DATATREE_ROOT)
    {
        return $GLOBALS['trean_shares']->extractOneLevel($parent);
    }

    function getCategoryParent($category)
    {
        return $GLOBALS['trean_shares']->getParent($category);
    }

    function getName($category)
    {
        return $GLOBALS['trean_shares']->getName($category);
    }

    /**
     * Generates the body of a &lt;select&gt; form input to select a
     * category. The &lt;select&gt; and &lt;/select&gt; tags are NOT included
     * in the output of this function.
     *
     * @param string $selected  The category to have selected by default.
     *                          Defaults to the first option in the list.
     *
     * @return string  A string containing <option> elements for each category
     *                 in the list.
     */
    function categorySelect($selected = DATATREE_ROOT, $perm = PERMS_SHOW,
                            $attributes = null, $parent = DATATREE_ROOT,
                            $allLevels = true, $from = 0, $count = 0,
                            $new = false)
    {
        global $trean_shares;
        require_once 'Horde/Tree.php';
        $category_tree = &Trean::listCategories($perm, $attributes, $parent,
                                                $allLevels, $from, $count);
        $tree = &Horde_Tree::factory('category_tree', 'select');
        foreach ($category_tree as $node_id => $node_val) {
            $indent = substr_count($node_id, ':');
            $node_id = $node_val->getId();
            $parent_id = $trean_shares->getParent($node_val);
            $label = $node_val->get('name');

            /* Selected or not? */
            $params['selected'] = ($node_id == $selected) ? true : false;

            if ($parent_id == DATATREE_ROOT) {
                $parent_id = null;
            }

            /* Add the node and add the node params. */
            $tree->addNode($node_id, $parent_id, $label,
                           $indent, true, $params);
        }

        $select = '';
        if ($new) {
            $select .= '<option value="">----</option>' .
                '<option value="*new*">' . _("New Category") . '</option>';
        }
        return $select . $tree->renderTree();
    }

    /**
     * Returns the specified permission for the current user.
     *
     * @param string $permission  A permission, currently only 'max_categories'
     *                            and 'max_bookmarks'.
     *
     * @return mixed  The value of the specified permission.
     */
    function hasPermission($permission)
    {
        global $perms;

        if (!$perms->exists('trean:' . $permission)) {
            return true;
        }

        $allowed = $perms->getPermissions('trean:' . $permission);
        if (is_array($allowed)) {
            switch ($permission) {
            case 'max_categories':
            case 'max_bookmarks':
                $allowed = array_reduce($allowed, create_function('$a, $b', 'return max($a, $b);'), 0);
                break;
            }
        }

        return $allowed;
    }

    function menu()
    {
        require_once 'Horde/Template.php';
        $t = &new Horde_Template();
        $t->set('forminput', Util::formInput());

            $t->set('accesskey', $GLOBALS['prefs']->getValue('widget_accesskey') ? Horde::getAccessKey(_("Open Fo_lder")) : '', true);
            $t->set('flist', IMP::flistSelect('', true, array(), $GLOBALS['imp']['mailbox'], false, false, true));

            $menu_view = $GLOBALS['prefs']->getValue('menu_view');
            $link = Horde::link('#', '', '', '', 'folderSubmit(true); return false;');
            $t->set('flink', sprintf('<ul><li>%s%s<br />%s</a></li></ul>', $link, ($menu_view != 'text') ? Horde::img('folders/folder_open.png') : '', ($menu_view != 'icon') ? Horde::highlightAccessKey(_("Open Fo_lder"), $t->get('accesskey')) : ''));

        $t->set('menu_string', Trean::getMenu('string'));

        echo $t->fetch(TREAN_TEMPLATES . '/menu.html');
    }

    /**
     * Builds Trean's list of menu items.
     */
    function getMenu($returnType = 'object')
    {
        global $conf, $registry;

        require_once 'Horde/Menu.php';

        $menu = &new Menu();
        $menu->add(Horde::applicationUrl('browse.php'), _("_Browse"), 'trean.png', null, null, null, basename($_SERVER['PHP_SELF']) == 'index.php' ? 'current' : null);
        if (!empty($conf['hooks']['permsdenied']) ||
            Trean::hasPermission('max_bookmarks') === true ||
            Trean::hasPermission('max_bookmarks') > $GLOBALS['trean_shares']->countBookmarks()) {
            $menu->add(Horde::applicationUrl('add.php'), _("_New Bookmark"), 'add.png');
        }
        $menu->add(Horde::applicationUrl('search.php'), _("_Search"), 'search.png', $registry->getImageDir('horde'));
        $menu->add(Horde::applicationUrl('reports.php'), _("_Reports"), 'reports.png');

        /* Import/Export. */
        if ($conf['menu']['import_export']) {
            $menu->add(Horde::applicationUrl('data.php'), _("_Import/Export"), 'data.png', $registry->getImageDir('horde'));
        }

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

    /**
     * Returns the "Reason Phrase" associated with the given HTTP status code
     * according to rfc2616.
     */
    function HTTPStatus($status_code)
    {
        switch ($status_code) {
        case '100': return _("Continue");
        case '101': return _("Switching Protocols");
        case '200': return _("OK");
        case '201': return _("Created");
        case '202': return _("Accepted");
        case '203': return _("Non-Authoritative Information");
        case '204': return _("No Content");
        case '205': return _("Reset Content");
        case '206': return _("Partial Content");
        case '300': return _("Multiple Choices");
        case '301': return _("Moved Permanently");
        case '302': return _("Found");
        case '303': return _("See Other");
        case '304': return _("Not Modified");
        case '305': return _("Use Proxy");
        case '307': return _("Temporary Redirect");
        case '400': return _("Bad Request");
        case '401': return _("Unauthorized");
        case '402': return _("Payment Required");
        case '403': return _("Forbidden");
        case '404': return _("Not Found");
        case '405': return _("Method Not Allowed");
        case '406': return _("Not Acceptable");
        case '407': return _("Proxy Authentication Required");
        case '408': return _("Request Time-out");
        case '409': return _("Conflict");
        case '410': return _("Gone");
        case '411': return _("Length Required");
        case '412': return _("Precondition Failed");
        case '413': return _("Request Entity Too Large");
        case '414': return _("Request-URI Too Large");
        case '415': return _("Unsupported Media Type");
        case '416': return _("Requested range not satisfiable");
        case '417': return _("Expectation Failed");
        case '500': return _("Internal Server Error");
        case '501': return _("Not Implemented");
        case '502': return _("Bad Gateway");
        case '503': return _("Service Unavailable");
        case '504': return _("Gateway Time-out");
        case '505': return _("HTTP Version not supported");
        default: return '';
        }
    }

    /**
     * Returns an apropriate icon for the given bookmark.
     */
    function getFavicon($bookmark)
    {
        global $registry;

        // Initialize VFS.
        require_once 'VFS.php';
        $vfs_params = Horde::getVFSConfig('favicons');
        if (is_a($vfs_params, 'PEAR_Error')) {
            // Default to the protocol icon.
            $url = $bookmark->get('url');
            $protocol = substr($url, 0, strpos($url, '://'));
            return $registry->getImageDir() . '/protocol/' .
                (empty($protocol) ? 'http' : $protocol) . '.png';
        }

        $vfs = &VFS::singleton($vfs_params['type'], $vfs_params['params']);
        if ($bookmark->get('favicon') &&
            $vfs->exists('.horde/trean/favicons/', $bookmark->get('favicon'))) {
            return Util::addParameter(Horde::applicationUrl('favicon.php'),
                                      'bookmark_id', $bookmark->getId());
        } elseif ($bookmark->get('http-Content-Type')) {
            require_once 'Horde/MIME/Viewer.php';
            $icon = MIME_Viewer::getIcon($bookmark->get('http-Content-Type'));
            if ($icon) {
                return $icon;
            }
        }

        // Default to the protocol icon.
        $url = $bookmark->get('url');
        $protocol = substr($url, 0, strpos($url, '://'));
        return $registry->getImageDir() . '/protocol/' .
            (empty($protocol) ? 'http' : $protocol) . '.png';
    }

}
